const template = document.createElement("template");
template.innerHTML = `<div class="progress-full-container fadeout">
<div class="progress-container">
    <div class="centerPortion">R10</div>
    <div class="progress-spinner" style="--progress: 0;"></div>
</div>
<div class="progress-explainer explainer"></div>
</div>
<style>
:host {
    font-family: inherit;
}
.progress-container {
    width: 100px;
    height: 64px;
    position: relative;
    /* display: flex; */
    /* align-items: center; */
    /* justify-content: center; */
}

.progress-spinner {
    width: 100%;
    height: 100%;
    border-radius: 6px;
    background: conic-gradient(green var(--progress, 0deg),
            lightgrey 0deg);
    background: conic-gradient(from 0deg at 50% 50%, #fb6949, #fb3d70 var(--progress, 0deg), lightgrey 0deg);
    background: conic-gradient(from 0deg at 50% 50%, #ffd1dc, #a2d2ff, #bde0fe, #ffd1dc var(--progress, 0deg), lightgrey 0deg);
    /* transform: rotate(-90deg); */
    transition: background 0.1s cubic-bezier(.4, 0, .2, 1);
    /* Adjust the duration as needed for smoothness */
}


.centerPortion {
    width: 88px;
    height: 50px;
    border-radius: 4px;
    position: absolute;
    top: 7px;
    left: 6px;
    display: flex;
    align-items: center;
    justify-content: center;
    z-index: 10;
    background: white;
    font-family: sans-serif;
    font-size: 20px;
}

.progress-full-container {
    display: flex;
    align-items: center;
    color: #666;
}

.progress-explainer {
    font-size: 12px;
    margin-left: 6px;
}

.explainer {
    /* existing styles */
    transition: all 0.3s ease;
}

/* Animation class */
.explainer-change {
    animation: fadeChange 0.6s;
}

@keyframes fadeChange {
    0% {
        opacity: 1;
    }

    50% {
        opacity: 0;
    }

    100% {
        opacity: 1;
    }
}

.progress-full-container {
    padding-top:8px;
    transition: opacity 0.5s ease-out;
}

.fadeout {
    opacity: 0;
    transition: opacity 0.5s ease-out;
}
</style>`;

class ProgressComponent extends HTMLElement {
  constructor() {
    super();
    this.attachShadow({ mode: "open" });
    this.shadowRoot.appendChild(template.content.cloneNode(true));

    var intervalId,
      explainerIntervalId,
      progressCompletedIntervalId = null;
  }

  connectedCallback() {
    // Initialize your component or attach event listeners
    // this.startProgressing();
  }

  // Your JavaScript methods go here
  setProgress(progress) {
    if (progress >= 100) {
      clearInterval(this.intervalId);
      if (this.progressCompletedIntervalId == null) {
        this.progressCompletedIntervalId = setInterval(() => {
          let degreeTemp = this.shadowRoot
            .querySelector(".progress-spinner")
            .style.getPropertyValue("--progress");
          degreeTemp = degreeTemp.replace("deg", "");
          progress = degreeTemp / 3.6;
          progress += 10;
          // setProgress(progress);
          const degree = progress * 3.6;
          this.shadowRoot
            .querySelector(".progress-spinner")
            .style.setProperty("--progress", `${degree}deg`);
          if (progress > 100) {
            clearInterval(this.progressCompletedIntervalId);
            clearInterval(this.explainerIntervalId);
            setTimeout(() => {
              this.updateExplainerContent("Completed");
            }, 200);
            setTimeout(() => {
              this.shadowRoot
                .querySelector(".progress-full-container")
                .classList.add("fadeout");
              if (
                document.querySelector("#R10-ProgressComponent") != undefined
              ) {
                document.querySelector("#R10-ProgressComponent").style.height =
                  "0px";
                setTimeout(() => {
                  document.querySelector("#R10-ProgressComponent").remove();
                }, 500);
              }
            }, 1500); // Wait for 500ms before fading out
          }
        }, 20);
      }

      return;
    }
    const degree = progress * 3.6;
    this.shadowRoot
      .querySelector(".progress-spinner")
      .style.setProperty("--progress", `${degree}deg`);
  }

  startProgressing() {
    let progress = 0;
    const totalDuration = 14000; // Total duration in milliseconds
    const intervalTime = 100; // Interval time in milliseconds
    const slowDownStart = 50; // Progress percentage to start slowing down
    const slowDownMore = 75; // Progress percentage to start slowing down
    const stopAt = 95; // Progress percentage to stop

    var changesList = ["", ""];
    // var changesList = ["", "Break input into tokens", "Convert tokens to numerical forms", "Understand context and intent", "Formulate a response", "Convert response to text", "Provide response to user"];
    var univI = 0;

    //reset spinner
    this.progressCompletedIntervalId = null;
    this.shadowRoot.querySelector(".progress-explainer").innerHTML = "";
    this.shadowRoot
      .querySelector(".progress-spinner")
      .style.setProperty("--progress", `0deg`);

    this.explainerIntervalId = setInterval(() => {
      univI++;
      if (changesList[univI] == undefined) {
        clearInterval(this.explainerIntervalId);
      } else {
        this.updateExplainerContent(changesList[univI]);
      }
    }, 3000);

    this.shadowRoot
      .querySelector(".progress-full-container")
      .classList.remove("fadeout");

    this.intervalId = setInterval(() => {
      // Increment progress
      // progress = progress < slowDownAt ? progress + 1 : progress + 0.45; // Adjust the increment values as needed

      if (progress < slowDownStart) {
        // Faster increment before slowing down
        progress += 1.5;
      } else if (progress >= slowDownStart && progress < slowDownMore) {
        // Slower increment as it approaches 100%
        progress += 0.65;
      } else if (progress < stopAt) {
        // Slower increment as it approaches 100%
        progress += 0.35;
      }

      this.setProgress(progress);

      if (progress >= stopAt) {
        clearInterval(this.intervalId);
      }
    }, intervalTime);
  }

  updateExplainerContent(newContent) {
    const explainer = this.shadowRoot.querySelector(".explainer");
    explainer.classList.add("explainer-change");

    // Change content when opacity is 0
    setTimeout(() => {
      explainer.textContent = newContent;
    }, 300); // Half the duration of the animation (fade out point)

    // Remove the class at the end of the animation
    setTimeout(() => {
      explainer.classList.remove("explainer-change");
    }, 600); // Full duration of the animation
  }

  // ... other methods ...
}

// Define the new element
customElements.define("progress-component", ProgressComponent);
